#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
from datetime import datetime
from atlas_client.client import Atlas
from atlas_client.entity_management.s3.S3BucketManager import S3BucketManager
from atlas_client.entity_management.s3.S3ObjectManager import S3ObjectManager
from atlas_client.entity_management.s3.S3PsDirManager import S3PsDirManager
from atlas_s3_hook.S3MetadataClient import S3MetadataClient


class S3Hook:
    def __init__(self, s3_client: S3MetadataClient, atlas_client: Atlas):
        self.s3_end_point = s3_client.get_end_point()
        self.fs = s3_client.get_fs()
        self.s3_bucket_manager = S3BucketManager(atlas_client)
        self.s3_ps_dir_manager = S3PsDirManager(atlas_client)
        self.s3_object_manager = S3ObjectManager(atlas_client)

    def create_atlas_bucket(self, bucket_metadata: dict, bucket_description: str) -> None:
        entity_name = bucket_metadata['name']
        qualified_bucket_name = "s3://" + self.s3_end_point + "/" + entity_name
        domain = self.s3_end_point
        date = bucket_metadata['CreationDate']
        create_time_stamp = round(datetime.timestamp(date) * 1000)
        print("timestamp =", create_time_stamp)
        self.s3_bucket_manager.create_entity(entity_name, domain,
                                             qualified_bucket_name,
                                             bucket_description, create_time=create_time_stamp)

    def create_atlas_ps_dir(self, ps_dir_metadata: dict, ps_dir_description: str) -> None:
        names = ps_dir_metadata['name'].split("/")
        bucket_name = names[0]
        entity_name = "/".join(names[1:])
        qualified_bucket_name = "s3://" + self.s3_end_point + "/" + bucket_name
        qualified_entity_name = qualified_bucket_name + "/" + entity_name
        prefix = entity_name + "/"
        self.s3_ps_dir_manager.create_entity(entity_name, qualified_entity_name,
                                             qualified_bucket_name, prefix, description=ps_dir_description)

    def create_atlas_object(self, object_metadata: dict, owner: str, object_description: str) -> None:
        names = object_metadata['name'].split('/')
        entity_name = names[-1]
        qualified_entity_name = "s3://" + object_metadata['name']
        qualified_ps_dir_name = "s3://" + "/".join(names[:-1])
        ps_dir_prefix = "/".join(names[1:-1]) + "/"
        extension = str(os.path.splitext(entity_name)[1])[1:]
        date = object_metadata['LastModified']
        last_modified_stamp = round(datetime.timestamp(date) * 1000)
        size = object_metadata['size']
        self.s3_object_manager.create_entity(entity_name, qualified_entity_name,
                                             qualified_ps_dir_name, ps_dir_prefix, extension,
                                             owner,
                                             object_description, create_time=last_modified_stamp,
                                             update_time=last_modified_stamp, size=size)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from atlas_client.client import Atlas
from atlas_s3_hook.S3Hook import S3Hook
from atlas_s3_hook.S3MetadataClient import S3MetadataClient
from atlas_s3_hook.S3PathClass import S3PathClass


class S3Scanner:
    def __init__(self, s3_client: S3MetadataClient, atlas_client: Atlas, owner="unknown") -> None:
        self.s3_client = s3_client
        self.fs = s3_client.get_fs()
        self.default_owner = owner
        self.s3_atlas_hook = S3Hook(s3_client, atlas_client)

    def scan_path(self, path: str, description: str = "generated by s3 atlas hook") -> None:
        sub_path_list = self.fs.ls(path)
        for sub_path in sub_path_list:
            path_class = self.s3_client.get_class_from_path(sub_path)
            current_path_meta = self.s3_client.get_path_meta_data(sub_path)
            if path_class == S3PathClass.BUCKET:
                self.s3_atlas_hook.create_atlas_bucket(current_path_meta, description)
                # call dir scanner recursively
                self.scan_path(sub_path)
            elif path_class == S3PathClass.DIR:
                self.s3_atlas_hook.create_atlas_ps_dir(current_path_meta, description)
                self.scan_path(sub_path)
            elif path_class == S3PathClass.OBJECT:
                # avoid loading .keep file
                if sub_path.split("/")[-1] == ".keep":
                    pass
                else:
                    self.s3_atlas_hook.create_atlas_object(current_path_meta, self.default_owner, description)
            else:
                raise ValueError



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import s3fs
from s3fs import S3FileSystem
from atlas_s3_hook.S3PathClass import S3PathClass
from atlas_s3_hook.S3PathType import S3PathType
from atlas_s3_hook.Exceptions import S3ClientMissingArg
import os


class S3MetadataClient:
    def __init__(self, s3_end_point: str = None, s3_access_key: str = None, s3_secret_key: str = None,
                 s3_token: str = None):
        if s3_end_point is None:
            s3_end_point = os.getenv('AWS_S3_ENDPOINT')
        if s3_access_key is None:
            s3_access_key = os.getenv('AWS_ACCESS_KEY_ID')
        if s3_secret_key is None:
            s3_secret_key = os.getenv('AWS_SECRET_ACCESS_KEY')
        if s3_token is None:
            s3_token = os.getenv('AWS_SESSION_TOKEN')
        if s3_end_point and s3_access_key and s3_secret_key and s3_token:
            self.s3_end_point = s3_end_point
            self.fs = s3fs.S3FileSystem(client_kwargs={'endpoint_url': 'http://' + s3_end_point}, key=s3_access_key,
                                        secret=s3_secret_key,
                                        token=s3_token)
        else:
            raise S3ClientMissingArg

    def get_fs(self) -> S3FileSystem:
        return self.fs

    def get_end_point(self) -> str:
        return self.s3_end_point

    @staticmethod
    def get_class_from_entity_meta(entity_metadata: dict) -> S3PathClass:
        path_class = entity_metadata['StorageClass']
        if path_class == 'BUCKET':
            return S3PathClass.BUCKET
        elif path_class == 'DIRECTORY':
            return S3PathClass.DIR
        elif path_class == 'STANDARD':
            return S3PathClass.OBJECT
        else:
            raise ValueError

    @staticmethod
    def get_type_from__entity_meta(entity_metadata: dict) -> S3PathType:
        path_type = entity_metadata['type']
        if path_type == 'directory':
            return S3PathType.directory
        elif path_type == 'file':
            return S3PathType.file
        else:
            raise ValueError

    def get_path_meta_data(self, entity_path: str) -> dict:
        meta_data = self.fs.stat(entity_path)
        return meta_data

    def get_class_from_path(self, entity_path: str) -> S3PathClass:
        entity_metadata = self.get_path_meta_data(entity_path)
        return self.get_class_from_entity_meta(entity_metadata)

    def get_type_from_path(self, entity_path: str) -> S3PathType:
        entity_metadata = self.get_path_meta_data(entity_path)
        return self.get_type_from__entity_meta(entity_metadata)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

LOG = logging.getLogger('AtlasS3Hook')


class ClientError(Exception):
    """
    The base exception class for all exceptions this library raises.
    """
    message = 'Unknown Error'

    def __init__(self, message=None):
        self.message = message or self.__class__.message
        super(ClientError, self).__init__()

    def __str__(self):
        exception_message = "Unexpected client-side error: %s" % self.message
        LOG.error(exception_message)
        return exception_message


class S3ClientMissingArg(ClientError):
    def __init__(self):
        super().__init__(
            message="You need to provide a s3_end_point, s3_access_key, s3_secret_key, "
                    "s3_token to connect to a s3 sever")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import setuptools

from atlas_s3_hook import __version__ as version

here = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, here)

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'requirements.txt')
with open(requirements_path) as requirements_file:
    requirements = requirements_file.readlines()

setuptools.setup(
    name="atlass3hook",  # Replace with your own username
    version=version,
    author="Pengfei Liu",
    author_email="pengfei.liu@insee.fr",
    description="This atlas s3 hook uses s3fs package to gather the metadata of bucket, pseudo_dir and object, then"
                " it inserts these metadata into Atlas instances.",
    license='Apache License 2.0',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.lab.sspcloud.fr/pengfei/atlass3hook",
    # we need to indicate excitement which package will be published, otherwise import will raise module name not found
    packages=setuptools.find_packages(include=['atlas_s3_hook']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=requirements,
    python_requires='>=3.7',

)
